/* ============================================================
 *
 * This file is a part of the build-system-essentials project
 *
 * Copyright (C) 2012 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "MySimpleCCAComponent.h"

#include <iostream>
#include <string>

using namespace std;
using namespace cca;
using namespace rci;

namespace ccaessentials {
namespace library {

MySimpleCCAComponent::MySimpleCCAComponent(double gain)
        : cca::Component("Simple Controller"), cgain(gain) {
    IN = InputPort<JointAngles>::create();      // Input goal port
    registerPort("in", IN);
    FDB = InputPort<JointAngles>::create();     // Input feedback port
    registerPort("fdb", FDB);
    OUT = OutputPort<JointAngles>::create();    // Output control port
    registerPort("out", OUT);
}

MySimpleCCAComponent::~MySimpleCCAComponent() {
}

void MySimpleCCAComponent::onProcess() {
    if (!IN->empty() && !FDB->empty()) { // If we have a command and feedback
        InputPort<JointAngles>::DataPtr jaIN = IN->get();
        InputPort<JointAngles>::DataPtr jaFDB = FDB->get();
        
        // Calculate controller output
        
        OutputPort<JointAngles>::DataPtr jaOUT = jaFDB;
        for (unsigned joint = 0; joint < 7; ++joint) {
            // Calculate one controller step for each joint
            jaOUT->setValue(joint,
                jaFDB->rad(joint) + cgain * (jaIN->rad(joint) - jaFDB->rad(joint)));
        }
        
        OUT->publish(jaOUT);
    }

}

}
}

